import _isFunction from 'lodash/isFunction';
import _isArray from 'lodash/isArray';
import _last from 'lodash/last'; /*
                                  * This file is part of the nivo project.
                                  *
                                  * Copyright 2016-present, Raphaël Benitte.
                                  *
                                  * For the full copyright and license information, please view the LICENSE
                                  * file that was distributed with this source code.
                                  */

import { scaleQuantize } from 'd3-scale';
import { schemeBrBG, schemePRGn, schemePiYG, schemePuOr, schemeRdBu, schemeRdGy, schemeRdYlBu, schemeRdYlGn, schemeSpectral, schemeBlues, schemeGreens, schemeGreys, schemeOranges, schemePurples, schemeReds, schemeBuGn, schemeBuPu, schemeGnBu, schemeOrRd, schemePuBuGn, schemePuBu, schemePuRd, schemeRdPu, schemeYlGnBu, schemeYlGn, schemeYlOrBr, schemeYlOrRd } from 'd3-scale-chromatic';

var quantizeColorScales = {
    brownBlueGreen: _last(schemeBrBG),
    purpleRedGreen: _last(schemePRGn),
    pinkYellowGreen: _last(schemePiYG),
    purpleOrange: _last(schemePuOr),
    redBlue: _last(schemeRdBu),
    redGrey: _last(schemeRdGy),
    redYellowBlue: _last(schemeRdYlBu),
    redYellowGreen: _last(schemeRdYlGn),
    spectral: _last(schemeSpectral),

    // Sequential (Single Hue)
    blues: _last(schemeBlues),
    greens: _last(schemeGreens),
    greys: _last(schemeGreys),
    oranges: _last(schemeOranges),
    purples: _last(schemePurples),
    reds: _last(schemeReds),

    // Sequential (Multi-Hue)
    blueGreen: _last(schemeBuGn),
    bluePurple: _last(schemeBuPu),
    greenBlue: _last(schemeGnBu),
    orangeRed: _last(schemeOrRd),
    purpleBlueGreen: _last(schemePuBuGn),
    purpleBlue: _last(schemePuBu),
    purpleRed: _last(schemePuRd),
    redPurple: _last(schemeRdPu),
    yellowGreenBlue: _last(schemeYlGnBu),
    yellowGreen: _last(schemeYlGn),
    yellowOrangeBrown: _last(schemeYlOrBr),
    yellowOrangeRed: _last(schemeYlOrRd)
};

export var quantizeColorScalesKeys = Object.keys(quantizeColorScales);

export var guessQuantizeColorScale = function guessQuantizeColorScale(colors) {
    if (_isFunction(colors)) return colors;

    if (quantizeColorScales[colors]) {
        // use predefined d3 quantize color scale
        return scaleQuantize().range(quantizeColorScales[colors]);
    }

    if (_isArray(colors)) {
        return scaleQuantize().range(colors);
    }

    // just use provided value,
    // all elements will have identical color
    return function (d) {
        return colors;
    };
};